/* ----------------------------------------------------------------------
 * milo/identify.c -- Identify the system we're running on
 *
 * Copyright (C) 1994 by Waldorf Electronics,
 * written by Andreas Busse
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License. See the file COPYING in the main directory of this archive for more
 * details.
 * ---------------------------------------------------------------------- */

#include <asm/bootinfo.h>
#include <asm/mipsregs.h>
#include <linux/string.h>
#include <arc/arc.h>
#include <identify.h>

extern struct bootinfo bi;

unsigned int icache_size     = 0; /* FIXME: cachectl.S needs these */
unsigned int icache_linesize = 0;
unsigned int dcache_size     = 0;
unsigned int dcache_linesize = 0;
unsigned int scache_size     = 0;
unsigned int scache_linesize = 0;

/* ----------------------------------------------------------------------
 * identify()
 * ---------------------------------------------------------------------- */

void identify()
{
  char *VendorId;		/* ARC vendor ID */
  char *ProductId;		/* ARC product ID */
  char *SystemId;		/* ARC system ID */
  int  cpuId;
  int  cpuConfiguration;
  int i;
  
  /*
   * Get system information from ARC BIOS 
   */

  VendorId = getVendorId();
  ProductId = getProductId();
  printf("ARC Vendor  ID\t\t: [%s]\r\n",VendorId);
  printf("ARC Product ID\t\t: [");
  for(i=0; i<8; i++)
    printf("%02x",(unsigned char)ProductId[i]);
  printf("]\r\n");
  SystemId = getSystemId();
  printf("ARC System  ID\t\t: [%s]\r\n",SystemId);

  /*
   * Get CPU Info
   */

  printf("CPU Type\t\t: [%s]\r\n",getCpuType());
  cpuId = read_32bit_cp0_register(CP0_PRID);
  printf("CPU ID\t\t\t: [%02x:%x.%x]\r\n",
	 cpuId >> 8,(cpuId & 0xf0) >> 4,cpuId & 0x0f);

  /*
   * Get Cache Sizes
   */

  cpuConfiguration = read_32bit_cp0_register(CP0_CONFIG);
  icache_size = bi.icache_size = 1 << (12 + ((cpuConfiguration & 0xe00) >> 9));
  dcache_size = bi.dcache_size = 1 << (12 + ((cpuConfiguration & 0x1c0) >> 6));
  icache_linesize = bi.icache_linesize = 16 << ((cpuConfiguration & 0x20) >> 5);
  dcache_linesize = bi.dcache_linesize = 16 << ((cpuConfiguration & 0x10) >> 4);
  
  printf("Primary ICache Size\t: [0x%4x bytes]\r\n",icache_size);
  printf("Primary DCache Size\t: [0x%4x bytes]\r\n",dcache_size);

#if 0  
  printf("Kseg0 coherency algorithm  : %d\r\n",cpuConfiguration & 0x7);
  printf("Update on store conditional: %d\r\n",(cpuConfiguration & 0x8) >> 3);
  printf("Primary DCache line size   : %d\r\n",(cpuConfiguration & 0x10) >> 4);
  printf("Primary ICache line size   : %d\r\n",(cpuConfiguration & 0x20) >> 5);
  printf("Primary DCache size        : %d\r\n",(cpuConfiguration & 0x1c0) >> 6);
  printf("Primary ICache size        : %d\r\n",(cpuConfiguration & 0xe00) >> 9);
  printf("Block ordering             : %d\r\n",(cpuConfiguration & 0x2000) >> 13);
  printf("ECC mode                   : %d\r\n",(cpuConfiguration & 0x4000) >> 14);
  printf("Endianess (0=little)       : %d\r\n",(cpuConfiguration & 0x8000) >> 15);
#endif

  printf("Secondary Cache Size\t: ");
  if ((bi.scache_size = getSecondaryCacheSize()) != SIZE_UNKNOWN)
    printf("[0x%4x]\r\n",bi.scache_size);
  else
    printf("[unknown]\r\n");
  
  printf("ARC System identified as: ");
  if (!strcmp("PICA-61",SystemId))
    {
      printf("ACER PICA-61\r\n");
      bi.machtype        = MACH_ACER_PICA_61;
      bi.cputype         = CPU_R4400PC;
      bi.memlower        = 0x80000000;
      bi.memupper        = 0x80800000;
      bi.scache_size     = 512*1024;
      bi.dma_cache       = 0;
      bi.vram_base       = 0xe08b8000; /* for 80x25, 640x480 */
    }
  else if (!strcmp("Deskstation Tyne",SystemId))
    { /* NOT CHECKED !!! Andy */
      printf("DeskStation Tyne\r\n");
      bi.machtype        = MACH_DESKSTATION_TYNE;
      bi.cputype         = CPU_R4600;
      bi.memlower        = 0x80000000;
      bi.memupper        = 0x80800000;
      bi.scache_size     = 512*1024;
      bi.dma_cache       = 512*1024;
      bi.vram_base       = 0xe10a0000;
    }
  else
    {
      printf(" Not identified -- using default parameters\r\n");
      bi.machtype        = MACH_UNKNOWN;
      bi.cputype         = CPU_R4000PC;
      bi.memlower        = 0x80000000;
      bi.memupper        = 0x80800000;
      bi.scache_size     = 0;
      bi.dma_cache       = 0;
    }
}

